%% Statistics for FiPIt model solutions, Mendoza and Villalvazo 2018
% Code for Coomputing Long-run Moments
% (C.) Enrique G. Mendoza & Sergio Villalvazo, 2018
clear all
close all
clc

tMyDir      = pwd;
tIdcs       = strfind(tMyDir,'\');
tMyDirUp    = tMyDir(1:tIdcs(end)-1);
tMFilesPath = [tMyDirUp, '\MFiles\'];
addpath(tMFilesPath);

%load solFiPIt_SS.mat    % For the constrained model (Sudden Stop)
load solFiPIt_RBC.mat   % For the unconstrained model (RBC)

% Mean
nMeanGDP    = sum(sum(sum(mGDP.*mErgDist)));
nMeanC      = sum(sum(sum(mC.*mErgDist)));
nMeanI      = sum(sum(sum(mInv.*mErgDist)));
nMeanNX_GDP = sum(sum(sum(mNX_GDP.*mErgDist)));
nMeanK      = sum(sum(sum(mCapital.*mErgDist)));
nMeanB_GDP  = sum(sum(sum(mB_GDP.*mErgDist)));
nMeanB      = sum(sum(sum(mBond.*mErgDist)));
nMeanQK     = sum(sum(sum(mPriceKGuess.*mErgDist)));
nMeanLev    = sum(sum(sum(mLev.*mErgDist)));
nMeanV      = sum(sum(sum(mV.*mErgDist)));
nMeanWC     = sum(sum(sum(mWC.*mErgDist)));
nMeanMu     = sum(sum(sum(mMu.*mErgDist)));

% Std Dev
nSigGDP    = (sum(sum(sum( ((mGDP - sum(sum(sum(mGDP.*mErgDist)))).^2.*mErgDist)))))^0.5 ;
nSigC      = (sum(sum(sum( ((mC - sum(sum(sum(mC.*mErgDist)))).^2.*mErgDist)))))^0.5 ;
nSigI      = (sum(sum(sum( ((mInv - sum(sum(sum(mInv.*mErgDist)))).^2.*mErgDist)))))^0.5 ;
nSigNX_GDP = (sum(sum(sum( ((mNX_GDP - sum(sum(sum(mNX_GDP.*mErgDist)))).^2.*mErgDist)))))^0.5 ;
nSigK      = (sum(sum(sum( ((mCapital - sum(sum(sum(mCapital.*mErgDist)))).^2.*mErgDist)))))^0.5 ;
nSigB_GDP  = (sum(sum(sum( ((mB_GDP - sum(sum(sum(mB_GDP.*mErgDist)))).^2.*mErgDist)))))^0.5 ;
nSigB      = (sum(sum(sum( ((mBond - sum(sum(sum(mBond.*mErgDist)))).^2.*mErgDist)))))^0.5 ;
nSigQK     = (sum(sum(sum( ((mPriceK - sum(sum(sum(mPriceK.*mErgDist)))).^2.*mErgDist)))))^0.5 ;
nSigLev    = (sum(sum(sum( ((mLev - sum(sum(sum(mLev.*mErgDist)))).^2.*mErgDist)))))^0.5 ;
nSigV      = (sum(sum(sum( ((mV - sum(sum(sum(mV.*mErgDist)))).^2.*mErgDist)))))^0.5 ;
nSigWC     = (sum(sum(sum( ((mWC - sum(sum(sum(mWC.*mErgDist)))).^2.*mErgDist)))))^0.5 ;
nSigMu     = (sum(sum(sum( ((mMu - sum(sum(sum(mMu.*mErgDist)))).^2.*mErgDist)))))^0.5 ;

% Corr with GDP
nRhoGDP_C  = ((sum(sum(sum(   ((mGDP - sum(sum(sum(mGDP.*mErgDist)))).*(mC - sum(sum(sum(mC.*mErgDist))))).*mErgDist   )))))/( ...
    (sum(sum(sum(   ((mGDP - sum(sum(sum(mGDP.*mErgDist)))).^2.*mErgDist)))))^0.5 * (sum(sum(sum(   ((mC - sum(sum(sum(mC.*mErgDist)))).^2.*mErgDist)))))^0.5 );
nRhoGDP_I  = ((sum(sum(sum(   ((mGDP - sum(sum(sum(mGDP.*mErgDist)))).*(mInv - sum(sum(sum(mInv.*mErgDist))))).*mErgDist   )))))/( ...
    (sum(sum(sum(   ((mGDP - sum(sum(sum(mGDP.*mErgDist)))).^2.*mErgDist)))))^0.5 * (sum(sum(sum(   ((mInv - sum(sum(sum(mInv.*mErgDist)))).^2.*mErgDist)))))^0.5 );
nRhoGDP_NX_GDP = ((sum(sum(sum(   ((mGDP - sum(sum(sum(mGDP.*mErgDist)))).*(mNX./mGDP - sum(sum(sum(mNX_GDP.*mErgDist))))).*mErgDist   )))))/( ...
    (sum(sum(sum(   ((mGDP - sum(sum(sum(mGDP.*mErgDist)))).^2.*mErgDist)))))^0.5 * (sum(sum(sum(   ((mNX_GDP - sum(sum(sum(mNX_GDP.*mErgDist)))).^2.*mErgDist)))))^0.5 );
nRhoGDP_K  = ((sum(sum(sum(   ((mGDP - sum(sum(sum(mGDP.*mErgDist)))).*(mCapital - sum(sum(sum(mCapital.*mErgDist))))).*mErgDist   )))))/( ...
    (sum(sum(sum(   ((mGDP - sum(sum(sum(mGDP.*mErgDist)))).^2.*mErgDist)))))^0.5 * (sum(sum(sum(   ((mCapital - sum(sum(sum(mCapital.*mErgDist)))).^2.*mErgDist)))))^0.5 );
nRhoGDP_B_GDP  = ((sum(sum(sum(   ((mGDP - sum(sum(sum(mGDP.*mErgDist)))).*(mB_GDP - sum(sum(sum(mB_GDP.*mErgDist))))).*mErgDist   )))))/( ...
    (sum(sum(sum(   ((mGDP - sum(sum(sum(mGDP.*mErgDist)))).^2.*mErgDist)))))^0.5 * (sum(sum(sum(   ((mB_GDP - sum(sum(sum(mB_GDP.*mErgDist)))).^2.*mErgDist)))))^0.5 );
nRhoGDP_B  = ((sum(sum(sum(   ((mGDP - sum(sum(sum(mGDP.*mErgDist)))).*(mBond - sum(sum(sum(mBond.*mErgDist))))).*mErgDist   )))))/( ...
    (sum(sum(sum(   ((mGDP - sum(sum(sum(mGDP.*mErgDist)))).^2.*mErgDist)))))^0.5 * (sum(sum(sum(   ((mBond - sum(sum(sum(mB_GDP.*mErgDist)))).^2.*mErgDist)))))^0.5 );
nRhoGDP_QK  = ((sum(sum(sum(   ((mGDP - sum(sum(sum(mGDP.*mErgDist)))).*(mPriceK - sum(sum(sum(mPriceK.*mErgDist))))).*mErgDist   )))))/( ...
    (sum(sum(sum(   ((mGDP - sum(sum(sum(mGDP.*mErgDist)))).^2.*mErgDist)))))^0.5 * (sum(sum(sum(   ((mPriceK - sum(sum(sum(mPriceK.*mErgDist)))).^2.*mErgDist)))))^0.5 );
nRhoGDP_Lev  = ((sum(sum(sum(   ((mGDP - sum(sum(sum(mGDP.*mErgDist)))).*(mLev - sum(sum(sum(mLev.*mErgDist))))).*mErgDist   )))))/( ...
    (sum(sum(sum(   ((mGDP - sum(sum(sum(mGDP.*mErgDist)))).^2.*mErgDist)))))^0.5 * (sum(sum(sum(   ((mLev - sum(sum(sum(mLev.*mErgDist)))).^2.*mErgDist)))))^0.5 ) ;
nRhoGDP_V  = ((sum(sum(sum(   ((mGDP - sum(sum(sum(mGDP.*mErgDist)))).*(mV - sum(sum(sum(mV.*mErgDist))))).*mErgDist   )))))/( ...
    (sum(sum(sum(   ((mGDP - sum(sum(sum(mGDP.*mErgDist)))).^2.*mErgDist)))))^0.5 * (sum(sum(sum(   ((mV - sum(sum(sum(mV.*mErgDist)))).^2.*mErgDist)))))^0.5 ) ;
nRhoGDP_WC  = ((sum(sum(sum(   ((mGDP - sum(sum(sum(mGDP.*mErgDist)))).*(mWC - sum(sum(sum(mWC.*mErgDist))))).*mErgDist   )))))/( ...
    (sum(sum(sum(   ((mGDP - sum(sum(sum(mGDP.*mErgDist)))).^2.*mErgDist)))))^0.5 * (sum(sum(sum(   ((mWC - sum(sum(sum(mWC.*mErgDist)))).^2.*mErgDist)))))^0.5 ) ;
nRhoGDP_Mu = ((sum(sum(sum(   ((mGDP - sum(sum(sum(mGDP.*mErgDist)))).*(mMu - sum(sum(sum(mMu.*mErgDist))))).*mErgDist   )))))/( ...
    (sum(sum(sum(   ((mGDP - sum(sum(sum(mGDP.*mErgDist)))).^2.*mErgDist)))))^0.5 * (sum(sum(sum(   ((mMu - sum(sum(sum(mMu.*mErgDist)))).^2.*mErgDist)))))^0.5 );

% Autocorrelation
nRhoGDP    = 0.0;
nRhoC      = 0.0;
nRhoI      = 0.0;
nRhoNX_GDP = 0.0;
nRhoK      = 0.0;
nRhoB_GDP  = 0.0;
nRhoB      = 0.0;
nRhoQK     = 0.0;
nRhoLev    = 0.0;
nRhoV      = 0.0;
nRhoWC     = 0.0;
nRhoMu     = 0.0;

for iBond = 1:nBondGrid
    for iCapital = 1:nCapitalGrid
        for iShock = 1:nShockGrid
            for iShockP = 1:nShockGrid 
                
                % Interpolation Procedure
                kX = mCapitalPrime(iBond,iCapital,iShock);
                bX = mBondPrimeGuess(iBond,iCapital,iShock);
                ikA = find(kX >= vCapitalGrid,1,'last');
                ibA = find(bX >= vBondGrid,1,'last');            
                if isempty(ikA) == 1 
                    ikA = 1;                  
                end
                if isempty(ibA) == 1
                    ibA = 1;                
                end
                if ikA == nCapitalGrid
                    ikA = nCapitalGrid-1;               
                end
                if ibA == nBondGrid
                    ibA = nBondGrid-1;               
                end

                kA = vCapitalGrid(ikA);
                kB = vCapitalGrid(ikA+1);

                bA = vBondGrid(ibA);
                bC = vBondGrid(ibA+1);

                fA = mPriceKGuess(ibA,ikA,iShockP);
                fB = mPriceKGuess(ibA,ikA+1,iShockP);
                fC = mPriceKGuess(ibA+1,ikA,iShockP);
                fD = mPriceKGuess(ibA+1,ikA+1,iShockP);

                iQKp = fBiLinearInterpolation(kA, kB, bA, bC, fA, fB, fC, fD, kX, bX);
                iKpp = vCapitalGrid(iCapital)/pAdjC*(iQKp - 1 + pAdjC) ; 

                fA = mBondPrimeGuess(ibA,ikA,iShockP);
                fB = mBondPrimeGuess(ibA,ikA+1,iShockP);
                fC = mBondPrimeGuess(ibA+1,ikA,iShockP);
                fD = mBondPrimeGuess(ibA+1,ikA+1,iShockP);

                iBpp = fBiLinearInterpolation(kA, kB, bA, bC, fA, fB, fC, fD, kX, bX);
                
                fA = mGDP(ibA,ikA,iShockP);
                fB = mGDP(ibA,ikA+1,iShockP);
                fC = mGDP(ibA+1,ikA,iShockP);
                fD = mGDP(ibA+1,ikA+1,iShockP);

                iGDPp = fBiLinearInterpolation(kA, kB, bA, bC, fA, fB, fC, fD, kX, bX);
                
                fA = mC(ibA,ikA,iShockP);
                fB = mC(ibA,ikA+1,iShockP);
                fC = mC(ibA+1,ikA,iShockP);
                fD = mC(ibA+1,ikA+1,iShockP);

                iCp = fBiLinearInterpolation(kA, kB, bA, bC, fA, fB, fC, fD, kX, bX);
                
                fA = mInv(ibA,ikA,iShockP);
                fB = mInv(ibA,ikA+1,iShockP);
                fC = mInv(ibA+1,ikA,iShockP);
                fD = mInv(ibA+1,ikA+1,iShockP);

                iIp = fBiLinearInterpolation(kA, kB, bA, bC, fA, fB, fC, fD, kX, bX);
                
                fA = mNX_GDP(ibA,ikA,iShockP);
                fB = mNX_GDP(ibA,ikA+1,iShockP);
                fC = mNX_GDP(ibA+1,ikA,iShockP);
                fD = mNX_GDP(ibA+1,ikA+1,iShockP);

                iNX_GDPp = fBiLinearInterpolation(kA, kB, bA, bC, fA, fB, fC, fD, kX, bX);
                
                fA = mB_GDP(ibA,ikA,iShockP);
                fB = mB_GDP(ibA,ikA+1,iShockP);
                fC = mB_GDP(ibA+1,ikA,iShockP);
                fD = mB_GDP(ibA+1,ikA+1,iShockP);

                iB_GDPp = fBiLinearInterpolation(kA, kB, bA, bC, fA, fB, fC, fD, kX, bX);
                
                fA = mLev(ibA,ikA,iShockP);
                fB = mLev(ibA,ikA+1,iShockP);
                fC = mLev(ibA+1,ikA,iShockP);
                fD = mLev(ibA+1,ikA+1,iShockP);

                iLevp = fBiLinearInterpolation(kA, kB, bA, bC, fA, fB, fC, fD, kX, bX);
                
                fA = mV(ibA,ikA,iShockP);
                fB = mV(ibA,ikA+1,iShockP);
                fC = mV(ibA+1,ikA,iShockP);
                fD = mV(ibA+1,ikA+1,iShockP);

                iVp = fBiLinearInterpolation(kA, kB, bA, bC, fA, fB, fC, fD, kX, bX);
                
                fA = mWC(ibA,ikA,iShockP);
                fB = mWC(ibA,ikA+1,iShockP);
                fC = mWC(ibA+1,ikA,iShockP);
                fD = mWC(ibA+1,ikA+1,iShockP);

                iWCp = fBiLinearInterpolation(kA, kB, bA, bC, fA, fB, fC, fD, kX, bX);
                
                fA = mMu(ibA,ikA,iShockP);
                fB = mMu(ibA,ikA+1,iShockP);
                fC = mMu(ibA+1,ikA,iShockP);
                fD = mMu(ibA+1,ikA+1,iShockP);

                iMup = fBiLinearInterpolation(kA, kB, bA, bC, fA, fB, fC, fD, kX, bX);
                
                % Interpolation Procedure Ends
                
                nRhoGDP = nRhoGDP + mErgDist(iBond,iCapital,iShock)*(...
                    (mGDP(iBond,iCapital,iShock) - nMeanGDP)*mTran(iShock,iShockP)*...
                    (iGDPp - nMeanGDP));
                
                nRhoC = nRhoC + mErgDist(iBond,iCapital,iShock)*(...
                    (mC(iBond,iCapital,iShock) - nMeanC)*mTran(iShock,iShockP)*...
                    (iCp - nMeanC));
                
                nRhoI = nRhoI + mErgDist(iBond,iCapital,iShock)*(...
                    (mInv(iBond,iCapital,iShock) - nMeanI)*mTran(iShock,iShockP)*...
                    (iIp - nMeanI));
                
                nRhoNX_GDP = nRhoNX_GDP + mErgDist(iBond,iCapital,iShock)*(...
                    (mNX_GDP(iBond,iCapital,iShock) - nMeanNX_GDP)*mTran(iShock,iShockP)*...
                    (iNX_GDPp - nMeanNX_GDP));
                
                nRhoK = nRhoK + mErgDist(iBond,iCapital,iShock)*(...
                    (mCapital(iBond,iCapital,iShock) - nMeanK)*mTran(iShock,iShockP)*...
                    (mCapitalPrime(iBond,iCapital,iShock) - nMeanK));
                
                nRhoB_GDP = nRhoB_GDP + mErgDist(iBond,iCapital,iShock)*(...
                    (mB_GDP(iBond,iCapital,iShock) - nMeanB_GDP)*mTran(iShock,iShockP)*...
                    (iB_GDPp - nMeanB_GDP));
                
                nRhoB = nRhoB + mErgDist(iBond,iCapital,iShock)*(...
                    (mBond(iBond,iCapital,iShock) - nMeanB)*mTran(iShock,iShockP)*...
                    (mBondPrimeGuess(iBond,iCapital,iShock) - nMeanB));
                
                nRhoQK = nRhoQK + mErgDist(iBond,iCapital,iShock)*(...
                    (mPriceK(iBond,iCapital,iShock) - nMeanQK)*mTran(iShock,iShockP)*...
                    (iQKp - nMeanI));
                
                nRhoLev = nRhoLev + mErgDist(iBond,iCapital,iShock)*(...
                    (mLev(iBond,iCapital,iShock) - nMeanLev)*mTran(iShock,iShockP)*...
                    (iLevp - nMeanLev));
                
                nRhoV = nRhoV + mErgDist(iBond,iCapital,iShock)*(...
                    (mV(iBond,iCapital,iShock) - nMeanV)*mTran(iShock,iShockP)*...
                    (iVp - nMeanV));
                
                nRhoWC = nRhoWC + mErgDist(iBond,iCapital,iShock)*(...
                    (mWC(iBond,iCapital,iShock) - nMeanWC)*mTran(iShock,iShockP)*...
                    (iWCp - nMeanWC));
                
                nRhoMu = nRhoMu + mErgDist(iBond,iCapital,iShock)*(...
                    (mMu(iBond,iCapital,iShock) - nMeanMu)*mTran(iShock,iShockP)*...
                    (iMup - nMeanMu));

            end
        end
    end
end

nRhoGDP    = nRhoGDP / (nSigGDP * nSigGDP);
nRhoC      = nRhoC / (nSigC * nSigC);
nRhoI      = nRhoI / (nSigI * nSigI);
nRhoNX_GDP = nRhoNX_GDP / (nSigNX_GDP * nSigNX_GDP);
nRhoK      = nRhoK / (nSigK * nSigK);
nRhoB_GDP  = nRhoB_GDP / (nSigB_GDP * nSigB_GDP);
nRhoB      = nRhoB / (nSigB * nSigB);
nRhoQK     = nRhoQK / (nSigQK * nSigQK);
nRhoLev    = nRhoLev / (nSigLev * nSigLev);
nRhoV      = nRhoV / (nSigV * nSigV);
nRhoWC     = nRhoWC / (nSigWC * nSigWC);
nRhoMu     = nRhoMu / (nSigMu * nSigMu);



fprintf('---------------------------');
fprintf('\n');
fprintf('Stats for FPI paper (Ergodic Distribution) \n');
fprintf('---------------------------');
fprintf('\n');
fprintf('Mean \n');
fprintf('gdp = %.3f \n',nMeanGDP);
fprintf('c = %.3f \n',nMeanC);
fprintf('i = %.3f \n',nMeanI);
fprintf('nx/gdp = %.3f \n',nMeanNX_GDP);
fprintf('k = %.3f \n',nMeanK);
fprintf('b/gdp = %.3f \n',nMeanB_GDP);
fprintf('q = %.3f \n',nMeanQK);
fprintf('lev = %.3f \n',nMeanLev);
fprintf('v = %.3f \n',nMeanV);
fprintf('work cap = %.3f \n',nMeanWC);
fprintf('\n');
fprintf('Standard Deviation \n');
fprintf('gdp = %.2f \n',nSigGDP/nMeanGDP*100);
fprintf('c = %.2f \n',nSigC/nMeanC*100);
fprintf('i = %.2f \n',nSigI/nMeanI*100);
fprintf('nx/gdp = %.2f \n',nSigNX_GDP*100);
fprintf('k = %.2f \n',nSigK/nMeanK*100);
fprintf('b/gdp = %.2f \n',nSigB_GDP*100);
fprintf('q = %.2f \n',nSigQK/nMeanQK*100);
fprintf('lev = %.2f \n',nSigLev*100);
fprintf('v = %.2f \n',nSigV/nMeanV*100);
fprintf('work cap = %.2f \n',nSigWC/nMeanWC*100);
fprintf('\n');
fprintf('Correlation with GDP \n');
fprintf('gdp = %.3f \n',1.000);
fprintf('c = %.3f \n',nRhoGDP_C);
fprintf('i = %.3f \n',nRhoGDP_I);
fprintf('nx/gdp = %.3f \n',nRhoGDP_NX_GDP);
fprintf('k = %.3f \n',nRhoGDP_K);
fprintf('b/gdp = %.3f \n',nRhoGDP_B_GDP);
fprintf('q = %.3f \n',nRhoGDP_QK);
fprintf('lev = %.3f \n',nRhoGDP_Lev);
fprintf('v = %.3f \n',nRhoGDP_V);
fprintf('work cap = %.3f \n',nRhoGDP_WC);
fprintf('\n');
fprintf('First Order Autocorrelation \n');
fprintf('gdp = %.3f \n',nRhoGDP);
fprintf('c = %.3f \n',nRhoC);
fprintf('i = %.3f \n',nRhoI);
fprintf('nx/gdp = %.3f \n',nRhoNX_GDP);
fprintf('k = %.3f \n',nRhoK);
fprintf('b/gdp = %.3f \n',nRhoB_GDP);
fprintf('q = %.3f \n',nRhoQK);
fprintf('lev = %.3f \n',nRhoLev);
fprintf('v = %.3f \n',nRhoV);
fprintf('work cap = %.3f \n',nRhoWC);
fprintf('\n');
fprintf('P(S.S) comes from script4_TableDiffAmpl \n');
fprintf('-------------------------------\n');
fprintf('Max Log10 Abs. Euler Error = %.2f \n',nMaxEEbond);
fprintf('Max Euler Error Pos: b = %1.0f, c = %1.0f, s = %1.0f \n',nMaxEEbondPosB,nMaxEEbondPosC,nMaxEEbondPosS);
fprintf('Mean Log10 Abs. Euler Error = %.2f \n',nMeanEEbond);
fprintf('\n');
fprintf('Max Log10 Abs. Euler Error = %.2f \n',nMaxEEcapital);
fprintf('Max Euler Error Pos: b = %1.0f, c = %1.0f, s = %1.0f \n',nMaxEEcapitalPosB,nMaxEEcapitalPosC,nMaxEEcapitalPosS);
fprintf('Mean Log10 Abs. Euler Error = %.2f \n',nMeanEEcapital);
fprintf('\n');
fprintf('Size (b,k) = (%.0f, %.0f)\n',nBondGrid, nCapitalGrid);
fprintf('Bond End Grid Points = (%.1f, %.1f)\n',nBondMin, nBondMax);
fprintf('Capital End Grid Points = (%.1f, %.1f)\n',nCapitalMin, nCapitalMax);
fprintf('\n');
fprintf('Time elapsed in seconds = %.0f \n',nTimeSeconds);
fprintf('Number of iterations = %.0f \n',nIter);



if pKappa < 1
    save solFiPIt_SS.mat 
else
    save solFiPIt_RBC.mat
end


