%% FiPIt: A Simple, Fast Global Method for Solving Models with Two
%% Endogenous States & Occasionally Binding Constraints
%% Mendoza & Villalvazo 2018
% Code for Constructing Polict Function Plots
% (C.) Enrique G. Mendoza & Sergio Villalvazo, 2018 

%% POLICY FUNCTION PLOTS

clear all
close all
clc

tMyDir      = pwd;
tIdcs       = strfind(tMyDir,'\');
tMyDirUp    = tMyDir(1:tIdcs(end)-1);
tMFilesPath = [tMyDirUp, '\MFiles\'];
addpath(tMFilesPath);

% Load mat file of SS model
 load 'solFiPIt_SS.mat'

mBpSS = mBondPrime;
mKpSS = mCapitalPrime;
mQKSS = mPriceK;
mMuSS = mMu;

clearvars -except mBpSS mKpSS mQKSS mMuSS pBondMinSSID

% Load mat file of RBC model
 load 'solFiPIt_RBC.mat'

mBp = mBondPrime(pBondMinSSID:end,:,:);
mKp = mCapitalPrime(pBondMinSSID:end,:,:);
mQK = mPriceK(pBondMinSSID:end,:,:);
mColor = ones(nBondGrid - pBondMinSSID+1,nCapitalGrid);

figure(1);clf;
set(figure(1),'PaperType','usletter','PaperOrientation','Landscape','PaperPosition',[0.1 0.1 11 8.5]);
colormap([1 0 0;0 0 1]);
surf(vBondGrid(pBondMinSSID:end),vCapitalGrid,mBpSS(:,:,7)',mColor'); hold on
surf(vBondGrid(pBondMinSSID:end),vCapitalGrid,mBp(:,:,7)'-10^-8,mColor'+1); hold on
axis([-250 100 600 900 -250 200])
xlabel('Bond')
ylabel('Capital')
zlabel('Bond Prime')
set(gca,'FontSize',20)
saveas(figure(1), 'BondPolicyBoth.pdf');

figure(1);clf;
set(figure(1),'PaperType','usletter','PaperOrientation','Landscape','PaperPosition',[0.1 0.1 11 8.5]);
colormap([1 0 0;0 0 1]);
surf(vBondGrid(pBondMinSSID:end),vCapitalGrid,mKpSS(:,:,7)'-10^-8,mColor'); hold on
surf(vBondGrid(pBondMinSSID:end),vCapitalGrid,mKp(:,:,7)',mColor'+1); hold on
axis([-250 100 600 900 350 900])
xlabel('Bond')
ylabel('Capital')
zlabel('Capital Prime')
set(gca,'FontSize',20)
saveas(figure(1), 'CapitalPolicyBoth.pdf');

figure(1);clf;
set(figure(1),'PaperType','usletter','PaperOrientation','Landscape','PaperPosition',[0.1 0.1 11 8.5]);
colormap([1 0 0;0 0 1]);
surf(vBondGrid(pBondMinSSID:end),vCapitalGrid,mQKSS(:,:,7)'-10^-8,mColor'); hold on
surf(vBondGrid(pBondMinSSID:end),vCapitalGrid,mQK(:,:,7)',mColor'+1); hold on
axis([-250 100 600 900 0 1.2])
xlabel('Bond')
ylabel('Capital')
zlabel('Capital Price')
set(gca,'FontSize',20)
saveas(figure(1), 'PriceKPolicyBoth.pdf');

figure(1);clf;
set(figure(1),'PaperType','usletter','PaperOrientation','Landscape','PaperPosition',[0.1 0.1 11 8.5]);
colormap([1 0 0;0 0 1]);
surf(vBondGrid(pBondMinSSID:end),vCapitalGrid,mMuSS(:,:,7)',mColor'); hold on
surf(vBondGrid(pBondMinSSID:end),vCapitalGrid,0*mMuSS(:,:,7)'-10^-8,mColor'+1); hold on
axis([-250 100 600 900 -10^-8 0.00048])
xlabel('Bond')
ylabel('Capital')
zlabel('Credit Constraint Multiplier')
set(gca,'FontSize',20)
saveas(figure(1), 'MuBoth.pdf');
    
    
    
    